package org.xbib.event.clock;

import java.util.concurrent.Callable;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.xbib.event.Event;
import org.xbib.event.bus.EventBus;
import org.xbib.event.common.EventManager;

public class ClockEventService implements Callable<Integer> {

    private static final Logger logger = Logger.getLogger(ClockEventService.class.getName());

    private final ClockEventManagerService manager;

    private final EventBus eventBus;

    private final String name;

    public ClockEventService(ClockEventManagerService manager,
                             EventBus eventBus,
                             String name) {
        this.manager = manager;
        this.eventBus = eventBus;
        this.name = name;
    }

    @Override
    public Integer call() {
        try {
            if (manager.getSuspended().contains(name)) {
                logger.log(Level.FINE, "clock event " + name + " suspended");
                return 1;
            } else {
                Event clockEvent = EventManager.eventBuilder()
                        .setType("clock")
                        .build();
                eventBus.post(clockEvent);
                return 0;
            }
        } catch (Throwable t) {
            logger.log(Level.WARNING, t.getMessage(), t);
            return 1;
        }
    }
}
