package org.xbib.event.path;

import org.xbib.event.bus.EventBus;
import org.xbib.event.common.EventManager;
import org.xbib.event.common.EventManagerService;
import org.xbib.settings.Settings;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;

public class FileFollowEventManagerService implements EventManagerService {

    private static final Logger logger = Logger.getLogger(FileFollowEventManagerService.class.getName());

    private Map<Future<?>, FileFollowEventService> eventServiceMap;

    public FileFollowEventManagerService() {
    }

    @Override
    public FileFollowEventManagerService init(EventManager eventManager) {
        Settings settings = eventManager.getSettings();
        EventBus eventBus = eventManager.getEventBus();
        ExecutorService executorService = eventManager.getExecutorService();
        this.eventServiceMap = new LinkedHashMap<>();
        for (Map.Entry<String, Settings> entry : settings.getGroups("event.filefollow").entrySet()) {
            Settings definition = entry.getValue();
            if (definition.getAsBoolean("enabled", true)) {
                String baseStr = definition.get("base");
                Objects.requireNonNull(baseStr);
                String patternStr = definition.get("pattern");
                Objects.requireNonNull(patternStr);
                try {
                    Path base = Paths.get(baseStr);
                    Pattern pattern = Pattern.compile(patternStr);
                    FileFollowEventService fileFollowEventService = new FileFollowEventService(definition, eventBus, base, pattern);
                    Future<?> future = executorService.submit(fileFollowEventService);
                    eventServiceMap.put(future, fileFollowEventService);
                    logger.log(Level.INFO, "file follow service " + entry.getKey() + " with base " + base + " and pattern " + pattern + " added");
                } catch (Exception e) {
                    logger.log(Level.SEVERE, "unable to create file follow service " + entry.getKey() + ", reason " + e.getMessage(), e);
                }
            }
        }
        return this;
    }

    @Override
    public void shutdown() throws IOException {
        for (Map.Entry<Future<?>, FileFollowEventService> entry : eventServiceMap.entrySet()) {
            try {
                entry.getValue().setKeepWatching(false);
                entry.getKey().cancel(true);
            } catch (Exception e) {
                logger.log(Level.SEVERE, e.getMessage(), e);
            }
        }
    }
}
