package org.xbib.event.generic;

import org.junit.jupiter.api.Test;
import org.xbib.event.Event;
import org.xbib.event.EventConsumer;
import org.xbib.event.bus.Subscribe;
import org.xbib.event.common.EventManager;
import org.xbib.event.common.GenericEventImpl;
import org.xbib.settings.Settings;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class GenericEventManagerTest {

    private static final Logger logger = Logger.getLogger(GenericEventManagerTest.class.getName());

    @Test
    void testGenericEvents() {
        Settings settings = Settings.settingsBuilder()
                .build();
        TestEventConsumer consumer = new TestEventConsumer();
        EventManager eventManager = EventManager.builder()
                .setSettings(settings)
                .register(consumer)
                .build();
        Event event = EventManager.eventBuilder()
                .setType("generic")
                .setListener(e -> logger.log(Level.INFO, "received event " + e))
                .build();
        eventManager.getGenericEventManagerService().post(event);
    }


    @Test
    void testGenericEventWithWaitForSubscriber() throws InterruptedException, ExecutionException {
        Settings settings = Settings.settingsBuilder()
                .build();
        TestEventConsumer consumer = new TestEventConsumer();
        EventManager eventManager = EventManager.builder()
                .setSettings(settings)
                .register(consumer)
                .build();
        CompletableFuture<Event> future = new CompletableFuture<>();
        Event event = EventManager.eventBuilder()
                .setType("generic")
                .setListener(e -> {
                    logger.log(Level.INFO, "received event " + e);
                    future.complete(e);
                })
                .build();
        eventManager.getGenericEventManagerService().post(event);
        Event e = future.get();
        logger.log(Level.INFO, "the event was received with result " + e + ", continuing");
    }

    @Test
    void testGenericEventWithWaitForAllConsumers() throws ExecutionException, InterruptedException, TimeoutException {
        Settings settings = Settings.settingsBuilder()
                .build();
        TestEventConsumer consumer1 = new TestEventConsumer();
        TestEventConsumer consumer2 = new TestEventConsumer();
        EventManager eventManager = EventManager.builder()
                .setSettings(settings)
                .register(consumer1)
                .register(consumer2)
                .loadEventConsumers()
                .build();
        CompletableFuture<Event> future = new CompletableFuture<>();
        Event event = EventManager.eventBuilder()
                .setType("generic")
                .setListener(e -> {
                    logger.log(Level.INFO, "received event " + e);
                    future.complete(e);
                })
                .build();
        eventManager.getGenericEventManagerService().post((GenericEventImpl) event, future);
        Event e = future.get(5L, TimeUnit.SECONDS);
        if (e != null) {
            logger.log(Level.INFO, "the event " + e + " was received by all consumers, continuing");
        }
    }

    private static class TestEventConsumer implements EventConsumer {

        TestEventConsumer() {
        }

        @Subscribe
        public void onEvent(GenericEventImpl event) {
            event.received();
        }
    }
}
